# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ports/port_msg.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ports/port_msg.proto',
  package='bess.pb',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x14ports/port_msg.proto\x12\x07\x62\x65ss.pb\"\x1a\n\x0bPCAPPortArg\x12\x0b\n\x03\x64\x65v\x18\x01 \x01(\t\"\xb5\x01\n\nPMDPortArg\x12\x10\n\x08loopback\x18\x01 \x01(\x08\x12\x11\n\x07port_id\x18\x02 \x01(\x04H\x00\x12\r\n\x03pci\x18\x03 \x01(\tH\x00\x12\x0e\n\x04vdev\x18\x04 \x01(\tH\x00\x12\x1d\n\x15vlan_offload_rx_strip\x18\x05 \x01(\x08\x12\x1e\n\x16vlan_offload_rx_filter\x18\x06 \x01(\x08\x12\x1c\n\x14vlan_offload_rx_qinq\x18\x07 \x01(\x08\x42\x06\n\x04port\"V\n\x11UnixSocketPortArg\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x1a\n\x12min_rx_interval_ns\x18\x02 \x01(\x03\x12\x17\n\x0f\x63onfirm_connect\x18\x03 \x01(\x08\"\xba\x01\n\x08VPortArg\x12\x0e\n\x06ifname\x18\x01 \x01(\t\x12\x10\n\x06\x64ocker\x18\x02 \x01(\tH\x00\x12\x17\n\rcontainer_pid\x18\x03 \x01(\x03H\x00\x12\x0f\n\x05netns\x18\x04 \x01(\tH\x00\x12\x10\n\x08rxq_cpus\x18\x05 \x03(\x03\x12\x0e\n\x06tx_tci\x18\x06 \x01(\x04\x12\x14\n\x0ctx_outer_tci\x18\x07 \x01(\x04\x12\x10\n\x08loopback\x18\x08 \x01(\x08\x12\x10\n\x08ip_addrs\x18\t \x03(\tB\x06\n\x04\x63pidb\x06proto3'
)




_PCAPPORTARG = _descriptor.Descriptor(
  name='PCAPPortArg',
  full_name='bess.pb.PCAPPortArg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dev', full_name='bess.pb.PCAPPortArg.dev', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=33,
  serialized_end=59,
)


_PMDPORTARG = _descriptor.Descriptor(
  name='PMDPortArg',
  full_name='bess.pb.PMDPortArg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='loopback', full_name='bess.pb.PMDPortArg.loopback', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port_id', full_name='bess.pb.PMDPortArg.port_id', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pci', full_name='bess.pb.PMDPortArg.pci', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vdev', full_name='bess.pb.PMDPortArg.vdev', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vlan_offload_rx_strip', full_name='bess.pb.PMDPortArg.vlan_offload_rx_strip', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vlan_offload_rx_filter', full_name='bess.pb.PMDPortArg.vlan_offload_rx_filter', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vlan_offload_rx_qinq', full_name='bess.pb.PMDPortArg.vlan_offload_rx_qinq', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='port', full_name='bess.pb.PMDPortArg.port',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=62,
  serialized_end=243,
)


_UNIXSOCKETPORTARG = _descriptor.Descriptor(
  name='UnixSocketPortArg',
  full_name='bess.pb.UnixSocketPortArg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='bess.pb.UnixSocketPortArg.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='min_rx_interval_ns', full_name='bess.pb.UnixSocketPortArg.min_rx_interval_ns', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='confirm_connect', full_name='bess.pb.UnixSocketPortArg.confirm_connect', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=245,
  serialized_end=331,
)


_VPORTARG = _descriptor.Descriptor(
  name='VPortArg',
  full_name='bess.pb.VPortArg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ifname', full_name='bess.pb.VPortArg.ifname', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='docker', full_name='bess.pb.VPortArg.docker', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='container_pid', full_name='bess.pb.VPortArg.container_pid', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='netns', full_name='bess.pb.VPortArg.netns', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rxq_cpus', full_name='bess.pb.VPortArg.rxq_cpus', index=4,
      number=5, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tx_tci', full_name='bess.pb.VPortArg.tx_tci', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tx_outer_tci', full_name='bess.pb.VPortArg.tx_outer_tci', index=6,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='loopback', full_name='bess.pb.VPortArg.loopback', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ip_addrs', full_name='bess.pb.VPortArg.ip_addrs', index=8,
      number=9, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='cpid', full_name='bess.pb.VPortArg.cpid',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=334,
  serialized_end=520,
)

_PMDPORTARG.oneofs_by_name['port'].fields.append(
  _PMDPORTARG.fields_by_name['port_id'])
_PMDPORTARG.fields_by_name['port_id'].containing_oneof = _PMDPORTARG.oneofs_by_name['port']
_PMDPORTARG.oneofs_by_name['port'].fields.append(
  _PMDPORTARG.fields_by_name['pci'])
_PMDPORTARG.fields_by_name['pci'].containing_oneof = _PMDPORTARG.oneofs_by_name['port']
_PMDPORTARG.oneofs_by_name['port'].fields.append(
  _PMDPORTARG.fields_by_name['vdev'])
_PMDPORTARG.fields_by_name['vdev'].containing_oneof = _PMDPORTARG.oneofs_by_name['port']
_VPORTARG.oneofs_by_name['cpid'].fields.append(
  _VPORTARG.fields_by_name['docker'])
_VPORTARG.fields_by_name['docker'].containing_oneof = _VPORTARG.oneofs_by_name['cpid']
_VPORTARG.oneofs_by_name['cpid'].fields.append(
  _VPORTARG.fields_by_name['container_pid'])
_VPORTARG.fields_by_name['container_pid'].containing_oneof = _VPORTARG.oneofs_by_name['cpid']
_VPORTARG.oneofs_by_name['cpid'].fields.append(
  _VPORTARG.fields_by_name['netns'])
_VPORTARG.fields_by_name['netns'].containing_oneof = _VPORTARG.oneofs_by_name['cpid']
DESCRIPTOR.message_types_by_name['PCAPPortArg'] = _PCAPPORTARG
DESCRIPTOR.message_types_by_name['PMDPortArg'] = _PMDPORTARG
DESCRIPTOR.message_types_by_name['UnixSocketPortArg'] = _UNIXSOCKETPORTARG
DESCRIPTOR.message_types_by_name['VPortArg'] = _VPORTARG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PCAPPortArg = _reflection.GeneratedProtocolMessageType('PCAPPortArg', (_message.Message,), {
  'DESCRIPTOR' : _PCAPPORTARG,
  '__module__' : 'ports.port_msg_pb2'
  # @@protoc_insertion_point(class_scope:bess.pb.PCAPPortArg)
  })
_sym_db.RegisterMessage(PCAPPortArg)

PMDPortArg = _reflection.GeneratedProtocolMessageType('PMDPortArg', (_message.Message,), {
  'DESCRIPTOR' : _PMDPORTARG,
  '__module__' : 'ports.port_msg_pb2'
  # @@protoc_insertion_point(class_scope:bess.pb.PMDPortArg)
  })
_sym_db.RegisterMessage(PMDPortArg)

UnixSocketPortArg = _reflection.GeneratedProtocolMessageType('UnixSocketPortArg', (_message.Message,), {
  'DESCRIPTOR' : _UNIXSOCKETPORTARG,
  '__module__' : 'ports.port_msg_pb2'
  # @@protoc_insertion_point(class_scope:bess.pb.UnixSocketPortArg)
  })
_sym_db.RegisterMessage(UnixSocketPortArg)

VPortArg = _reflection.GeneratedProtocolMessageType('VPortArg', (_message.Message,), {
  'DESCRIPTOR' : _VPORTARG,
  '__module__' : 'ports.port_msg_pb2'
  # @@protoc_insertion_point(class_scope:bess.pb.VPortArg)
  })
_sym_db.RegisterMessage(VPortArg)


# @@protoc_insertion_point(module_scope)
